package com.platform.util;
import java.io.UnsupportedEncodingException;
import java.security.SignatureException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.codec.digest.DigestUtils;

/**
 *类名：EpayMD5  				  <br/>
 *详细：EpayMD5签名类                                     <br/>
 *版本：1.0               		  <br/>				
 *日期：20130510				      <br/>
 *说明：根据参数和字符集获取签名          <br/> 
 **/
public class EpayMD5 {

	/**
     * 签名字符串
     * @param val 需要签名的字符串
     * @param key 密钥
     * @param input_charset 编码格式
     * @return 签名结果
     */
    public static String sign(String val, String key, String input_charset) {
    	val = val + key;
    	 String md5Hex = DigestUtils.md5Hex(getContentBytes(val, input_charset));
    	return md5Hex;
    }
    
    /**
     * 验证签名字符串
     * @param val 需要签名的字符串
     * @param sign 签名结果
     * @param key 密钥
     * @param input_charset 编码格式
     * @return true:验证通过，false:验证失败
     */
    public static boolean validateSign(String val, String sign, String key, String input_charset) {
    	val = val + key;
    	String mysign = DigestUtils.md5Hex(getContentBytes(val, input_charset));
    	if(mysign.equals(sign)) {
    		return true;
    	}else{
    		return false;
    	}
    }

    /**
     * @param content
     * @param charset
     * @return
     * @throws SignatureException
     * @throws UnsupportedEncodingException 
     */
    public static byte[] getContentBytes(String content, String charset) {
        if (charset == null || "".equals(charset)) {
            return content.getBytes();
        }
        if(!"utf-8".equalsIgnoreCase(charset))
        	new  UnsupportedEncodingException();
        try {
            return content.getBytes(charset);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("MD5签名出现错误,编码集不对,错误编码集是:" + charset);
        }
    }
    public static void main(String[] args) {
		System.out.println(DigestUtils.md5Hex(getContentBytes("123456", "UTF-8")));
	}
    
    /************************************************************************************************/
    public static String getrecharge(Map<String,String> params,String key,String charSet){
    	System.out.println("生成签名begin....");
		//过滤为空的数据
		params = paraFilter(params);
		//拼接参数 
		String val = createLinkString(params);
    	String text = val + key;
    	String mysign = DigestUtils.md5Hex(getContentBytes(text, "UTF-8"));
    	return mysign;
    }
    
    public static Map<String, String> paraFilter(Map<String, String> sArray) {

        Map<String, String> result = new HashMap<String, String>();

        if (sArray == null || sArray.size() <= 0) {
            return result;
        }

        for (String key : sArray.keySet()) {
            String value = sArray.get(key);
            if (value == null || value.equals("") || key.equalsIgnoreCase("sign")
                || key.equalsIgnoreCase("sign_type")) {
                continue;
            }
            result.put(key, value);
        }

        return result;
    }
    
    /** 
     * 把数组所有元素排序，并按照“参数=参数值”的模式用“&”字符拼接成字符串
     * @param params 需要排序并参与字符拼接的参数组
     * @return 拼接后字符串
     */
    public static String createLinkString(Map<String, String> params) {

        List<String> keys = new ArrayList<String>(params.keySet());
        Collections.sort(keys);

        String prestr = "";

        for (int i = 0; i < keys.size(); i++) {
            String key = keys.get(i);
            String value = params.get(key);

            if (i == keys.size() - 1) {//拼接时，不包括最后一个&字符
                prestr = prestr + key + "=" + value;
            } else {
                prestr = prestr + key + "=" + value + "&";
            }
        }

        return prestr;
    }
}